@@ -12,8 +12,8 @@ Tools and Utilities for debugging and helping with development of Editra.
 
 """
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: dev_tool.py 60454 2009-05-01 01:20:04Z CJP $"
-__revision__ = "$Revision: 60454 $"
+__svnid__ = "$Id: dev_tool.py 61502 2009-07-22 23:35:16Z CJP $"
+__revision__ = "$Revision: 61502 $"
 
 #-----------------------------------------------------------------------------#
 # Imports
@@ -27,12 +27,12 @@ import urllib2
 import webbrowser
 import codecs
 import locale
-import types
 import wx
 
 # Editra Libraries
 import ed_glob
 import ed_msg
+from ebmlib import IsUnicode
 
 #-----------------------------------------------------------------------------#
 # Globals
@@ -219,7 +219,7 @@ def DecodeString(string, encoding=None):
     if encoding is None:
         encoding = DEFAULT_ENCODING
 
-    if not isinstance(string, types.UnicodeType):
+    if not IsUnicode(string):
         try:
             rtxt = codecs.getdecoder(encoding)(string)[0]
         except Exception, msg:
@@ -417,10 +417,10 @@ class ErrorDialog(wx.Dialog):
             msg = "mailto:%s?subject=Error Report&body=%s"
             addr = "bugs@%s" % (ed_glob.HOME_PAGE.replace("http://", '', 1))
             if wx.Platform != '__WXMAC__':
-                msg = urllib2.quote(self.err_msg)
+                body = urllib2.quote(self.err_msg)
             else:
-                msg = self.err_msg
-            msg = msg % (addr, msg)
+                body = self.err_msg
+            msg = msg % (addr, body)
             msg = msg.replace("'", '')
             webbrowser.open(msg)
             self.Close()
