@@ -35,8 +35,8 @@ eclib.ctrlbox
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: finddlg.py 60840 2009-05-31 16:00:50Z CJP $"
-__revision__ = "$Revision: 60840 $"
+__svnid__ = "$Id: finddlg.py 61489 2009-07-21 15:54:54Z CJP $"
+__revision__ = "$Revision: 61489 $"
 
 __all__ = ["FindBox", "FindEvent", "FindPanel", "FindReplaceDlg",
            "MiniFindReplaceDlg", "AdvFindReplaceDlg",
@@ -113,6 +113,7 @@ ID_FIND_ALL = wx.NewId()
 ID_REPLACE_ALL = wx.NewId()
 ID_OPTION_CHANGE = wx.NewId()
 ID_CHOOSE_DIR = wx.NewId()
+ID_DIR_BOX = wx.NewId()
 
 _ = wx.GetTranslation
 
@@ -784,7 +785,7 @@ class FindPanel(wx.Panel):
 
         # Search Direction Box
         self._sizers['dir'] = wx.BoxSizer(wx.VERTICAL)
-        dbox = wx.StaticBox(self, label=_("Direction"))
+        dbox = wx.StaticBox(self, id=ID_DIR_BOX, label=_("Direction"))
         dboxsz = wx.StaticBoxSizer(dbox, wx.HORIZONTAL)
         dboxsz.AddMany([(wx.RadioButton(self, wx.ID_UP, _("Up")), 0),
                         ((20, 5), 0),
@@ -909,6 +910,11 @@ class FindPanel(wx.Panel):
         self.FindWindowById(ID_RECURSE).Enable(in_files)
         flags = self._fdata.GetFlags()
 
+        # Disable direction settings when searcing in file since they
+        # are ignored anyway.
+        for cid in (wx.ID_UP, wx.ID_DOWN, ID_DIR_BOX):
+            self.FindWindowById(cid).Enable(not in_files)
+
         # Only update visibility of file filter field if it is enabled
         if not (flags & AFR_NOFILTER):
             self.ShowFileFilters(in_files)
