@@ -15,15 +15,14 @@ text documents and files.
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: ed_search.py 60840 2009-05-31 16:00:50Z CJP $"
-__revision__ = "$Revision: 60840 $"
+__svnid__ = "$Id: ed_search.py 61516 2009-07-24 20:05:13Z CJP $"
+__revision__ = "$Revision: 61516 $"
 
 #--------------------------------------------------------------------------#
 # Imports
 import os
 import sys
 import re
-import types
 import wx
 
 # Local imports
@@ -43,6 +42,7 @@ _ = wx.GetTranslation
 #--------------------------------------------------------------------------#
 
 class EdSearchEngine(ebmlib.SearchEngine):
+    """Text searching engine"""
     def __init__(self, query, regex=True, down=True,
                  matchcase=True, wholeword=False):
         ebmlib.SearchEngine.__init__(self, query, regex, down,
@@ -58,11 +58,11 @@ class EdSearchEngine(ebmlib.SearchEngine):
 
         """
         fname = ed_txt.DecodeString(fname, sys.getfilesystemencoding())
-        if not isinstance(fname, types.UnicodeType):
+        if not ebmlib.IsUnicode(fname):
             fname = _("DECODING ERROR")
 
         match = ed_txt.DecodeString(match)
-        if not isinstance(match, types.UnicodeType):
+        if not ebmlib.IsUnicode(match):
             match = _("DECODING ERROR")
         else:
             match = u" " + match.lstrip()
@@ -80,7 +80,7 @@ class EdSearchEngine(ebmlib.SearchEngine):
 
     def SetQuery(self, query):
         """Set the query string"""
-        if isinstance(query, types.UnicodeType):
+        if ebmlib.IsUnicode(query):
             # Encode to UTF-8 as used internally by the stc
             query = query.encode('utf-8')
         super(EdSearchEngine, self).SetQuery(query)
@@ -128,19 +128,27 @@ class SearchController(object):
     def __del__(self):
         """Cleanup message handlers"""
         ed_msg.Unsubscribe(self._OnShowFindMsg)
+        if self._finddlg is not None:
+            self._finddlg.Destroy()
 
     def _CreateNewDialog(self, e_id):
         """Create and set the controlers find dialog
         @param e_id: Dialog Type Id
 
         """
+        # TODO: find out why parent is not a Window in some cases...
+        if not isinstance(self._parent, wx.Window):
+            parent = wx.GetApp().ActiveWindow()
+        else:
+            parent = self._parent
+
         if e_id == ed_glob.ID_FIND_REPLACE:
-            dlg = eclib.AdvFindReplaceDlg(self._parent, self._data,
-                                            (_("Find"), _("Find/Replace")),
-                                            eclib.AFR_STYLE_REPLACEDIALOG)
+            dlg = eclib.AdvFindReplaceDlg(parent, self._data,
+                                          (_("Find"), _("Find/Replace")),
+                                          eclib.AFR_STYLE_REPLACEDIALOG)
         elif e_id == ed_glob.ID_FIND:
-            dlg = eclib.AdvFindReplaceDlg(self._parent, self._data,
-                                            (_("Find"), _("Find/Replace")))
+            dlg = eclib.AdvFindReplaceDlg(parent, self._data,
+                                          (_("Find"), _("Find/Replace")))
         else:
             dlg = None
 
@@ -185,14 +193,15 @@ class SearchController(object):
                     return
                 self._finddlg.SetTransparent(240)
     #            self._finddlg.SetExtraStyle(wx.WS_EX_PROCESS_UI_UPDATES)
-                self._finddlg.Show()
             else:
-                # Dialog is open already so just update it
+                # Dialog has been created already so just update it
                 self._UpdateDialogState(ed_glob.ID_FIND)
-                self._finddlg.Raise()
 
             if 'lookin' in data:
                 self._finddlg.SetLookinPath(data.get('lookin'))
+
+            self._finddlg.Show()
+            self._finddlg.Raise()
             self._finddlg.SetFocus()
         else:
             return
@@ -497,11 +506,6 @@ class SearchController(object):
             # Save the most recent choices of search locations
             Profile_Set('SEARCH_LOC', choices)
             Profile_Set('SEARCH_FILTER', self._filters)
-
-            # Destroy it
-#            self._finddlg.Destroy()
-
-#        self._finddlg = None
         evt.Skip()
 
     def OnOptionChanged(self, evt):
@@ -522,7 +526,14 @@ class SearchController(object):
         if evt.IsRegEx() and self._engine is not None:
             match = self._engine.GetLastMatch()
             if match is not None:
-                value = match.expand(replacestring)
+                try:
+                    value = match.expand(replacestring)
+                except re.error, err:
+                    msg = _("Error in regular expression expansion."
+                            "The replace action cannot be completed.\n\n"
+                            "Error Message: %s") % err.message
+                    wx.MessageBox(msg, _("Replace Error"), wx.OK|wx.ICON_ERROR)
+                    return
             else:
                 value = replacestring
         else:
@@ -672,7 +683,14 @@ class SearchController(object):
         for match in reversed(matches):
             start, end = match.span()
             if isregex:
-                value = match.expand(rstring.encode('utf-8')).decode('utf-8')
+                try:
+                    value = match.expand(rstring.encode('utf-8')).decode('utf-8')
+                except re.error, err:
+                    msg = _("Error in regular expression expansion."
+                            "The replace action cannot be completed.\n\n"
+                            "Error Message: %s") % err.message
+                    wx.MessageBox(msg, _("Replace Error"), wx.OK|wx.ICON_ERROR)
+                    break
             else:
                 value = rstring
             stc.SetTargetStart(start)
