@@ -9,13 +9,14 @@
 """Unittests for EdFile"""
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: testEdFile.py 60505 2009-05-03 19:18:21Z CJP $"
-__revision__ = "$Revision: 60505 $"
+__svnid__ = "$Id: testEdFile.py 61520 2009-07-25 02:08:49Z CJP $"
+__revision__ = "$Revision: 61520 $"
 
 #-----------------------------------------------------------------------------#
 # Imports
 import wx
 import os
+import locale
 import types
 import unittest
 
@@ -26,18 +27,28 @@ import common
 import ed_txt
 import util
 import ebmlib
+import profiler
 
 #-----------------------------------------------------------------------------#
 
 class EdFileTest(unittest.TestCase):
     def setUp(self):
+        profiler.Profile_Set('ENCODING', locale.getpreferredencoding())
+
         self.app = common.EdApp(False)
         self.path = common.GetDataFilePath(u'test_read_utf8.txt')
         self.file = ed_txt.EdFile(self.path)
         self.mtime = ebmlib.GetFileModTime(self.path)
 
+        self.rpath = common.GetDataFilePath(u'embedded_nulls.txt')
+        self.rfile = ed_txt.EdFile(self.rpath)
+
+        self.ipath = common.GetDataFilePath(u'image_test.png')
+        self.img = ed_txt.EdFile(self.ipath)
+
     def tearDown(self):
         self.file.Close()
+        self.rfile.Close()
 
     #---- Tests ----#
     def testRead(self):
@@ -68,6 +79,22 @@ class EdFileTest(unittest.TestCase):
         """Test checking if file has a bom marker or not"""
         self.assertFalse(self.file.HasBom(), "File has a BOM")
 
+    def testIsRawBytes(self):
+        """Test reading a file that can't be properly encoded and was
+        read as raw bytes.
+
+        """
+        txt = self.file.Read()
+        self.assertTrue(ebmlib.IsUnicode(txt))
+        self.assertFalse(self.file.IsRawBytes())
+
+        txt = self.rfile.Read()
+        self.assertFalse(ebmlib.IsUnicode(txt))
+        self.assertTrue(self.rfile.IsRawBytes())
+
+        bytes = self.img.Read()
+        self.assertTrue(self.rfile.IsRawBytes())
+
     def testIsReadOnly(self):
         """Test if the file is read only or not"""
         self.assertFalse(self.file.IsReadOnly(), "File is readonly")
